﻿using AtmarkitReader.Common;

using System;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace AtmarkitReader
{
    sealed partial class App : Application
    {
        public App()
        {
            this.InitializeComponent();
            this.Suspending += OnSuspending;
        }

        // アプリがエンド・ユーザーにより正常に起動されたときに呼び出される。
        // 【パラメータ】args： 起動要求とプロセスの詳細を表すオブジェクト。
        protected override async void OnLaunched(LaunchActivatedEventArgs args)
        {
            // アプリがすでに実行中の場合は、アプリの初期化を繰り返さずに、
            // ウィンドウのアクティブ化のみを行う
            if (args.PreviousExecutionState == ApplicationExecutionState.Running)
            {
                Window.Current.Activate();
                return;
            }

            // ナビゲーション・コンテキストとして動作させるフレームを作成。
            // SuspensionManagerキー「AppFrame」とフレームを関連付ける
            var rootFrame = new Frame();
            SuspensionManager.RegisterFrame(rootFrame, "AppFrame");

            // アプリが終了していた場合は、保存されていたセッション状態を復元する
            if (args.PreviousExecutionState == ApplicationExecutionState.Terminated)
            {
                await SuspensionManager.RestoreAsync();
            }

            // ナビゲーション・スタックが復元されないときは、初期ページに遷移する
            if (rootFrame.Content == null)
            {
                if (!rootFrame.Navigate(typeof(ItemsPage), "AllGroups"))
                {
                    throw new Exception("初期ページの作成に失敗しました。");
                }
            }

            // フレームを現在のウィンドウに配置し、アクティブ化する
            Window.Current.Content = rootFrame;
            Window.Current.Activate();
        }

        // アプリの実行が中断されたときに呼び出される。
        // 【パラメータ】
        //  sender： 中断要求の送信元。
        //  e： 中断要求の詳細。
        private async void OnSuspending(object sender, SuspendingEventArgs e)
        {
            // 中断を保留してからセッション状態を保存し、最後に保留を解除する
            var deferral = e.SuspendingOperation.GetDeferral();
            await SuspensionManager.SaveAsync();
            deferral.Complete();
        }
    }
}
